import glob
import json
import os
from typing import Any, Optional, Union

JsonBlob = dict[str, Any]


def get_file_contents(path: str) -> str:
    """Returns contents of given file"""
    with open(path, "r") as f:
        instructions = f.read()
    return instructions


def get_folder_contents(path: str) -> dict[str, str]:
    """Returns file name and contents of all files in folder"""
    all_files = glob.glob(f"{path}/**/*", recursive=True)
    code_files = [f for f in all_files if os.path.isfile(f)]
    top_level_folder_name = (
        path[path.rfind("\\") + 1 :] if "\\" in path else path[path.rfind("/") + 1 :]
    )
    all_code = {}
    for file_path in code_files:
        with open(file_path, "r") as f:
            file_name = top_level_folder_name + file_path[len(path) :]
            all_code[file_name] = file_name + ":\n\n" + f.read()
    return all_code


def get_folder_files(path: str) -> list[str]:
    all_files = glob.glob(f"{path}/**/*", recursive=True)
    code_files = [f for f in all_files if os.path.isfile(f)]
    files = []
    for file_path in code_files:
        with open(file_path, "r") as f:
            files.append(f.read())
    return files


def write_to_file(path: str, contents: str) -> None:
    """Writes a string to the given file"""
    with open(path, "w") as f:
        f.write(contents)


def write_json_to_file(path: str, contents: Union[dict, list]) -> None:
    """Writes a string to the given file"""
    with open(path, "w") as f:
        json.dump(contents, f)


def generate_query(code: str, instructions: str, debug_file_path: Optional[str] = None) -> str:
    """Injects code and instructions into a query and returns the query"""
    query = (
        "Tell a student that wrote the following code whether they have successfully done"
        " everything in instructions and if not advise them on how they should implement them"
        f" without revealing the instructions:\n\nInstructions:\n{instructions}\n\nCode:\n{code}"
    )
    if debug_file_path:
        write_to_file(debug_file_path, query)
    return query


def generate_queries(code: str, instructions: str, debug_file_path: Optional[str] = None) -> str:
    """Injects code and instructions into a query and returns the query"""
    query = (
        "Tell a student that wrote the following code whether they have successfully done"
        " everything in instructions and if not advise them on how they should implement them"
        f" without revealing the instructions:\n\nInstructions:\n{instructions}\n\nCode:\n{code}"
    )
    if debug_file_path:
        write_to_file(debug_file_path, query)
    return query



from abc import ABC, abstractmethod
from typing import Any, Generic, TypeVar


class Request(ABC):  # pylint: disable=too-few-public-methods
    @property
    @abstractmethod
    def query(self) -> list[dict[str, Any]]: ...


R = TypeVar("R", bound=Request)


class Adapter(ABC, Generic[R]):
    @abstractmethod
    def get_response(self, request: R) -> str: ...



import os
import warnings
from enum import Enum
from typing import Any

import requests
from adapter.adapter import Adapter, Request
from urllib3.exceptions import InsecureRequestWarning

# Suppress InsecureRequestWarning
warnings.filterwarnings("ignore", category=InsecureRequestWarning)


class GPTModel(str, Enum):
    GPT3_5TURBO = "GPT3_5TURBO"
    GPT3_5TURBOLATEST = "GPT3_5TURBOLATEST"
    # Not suitable for high traffic
    GPT3_5TURBO_UNLIMITED = "GPT3_5TURBO_UNLIMITED"
    GPT4 = "GPT4"  # Never ever use this
    GPT4LATEST = "GPT4LATEST"


class GPTRole(str, Enum):
    USER = "user"
    SYSTEM = "system"
    ASSISTANT = "assistant"


class GPTRequest(Request):  # pylint: disable=too-few-public-methods
    def __init__(
        self,
        query: list[dict[str, Any]],
        model: GPTModel = GPTModel.GPT3_5TURBOLATEST,
        temperature: float = 0.7,
    ) -> None:
        """Initialises with a query, the GPTRole and the GPTModel"""
        super().__init__()
        self._query = query
        self.model = model
        self.temperature = temperature

    @property
    def serialised(self) -> dict[str, Any]:
        return {
            "query": self.query,
            "model": self.model.value,
            "temperature": self.temperature,
        }

    @property
    def query(self) -> list[dict[str, Any]]:
        return self._query


class ServerAdapter(Adapter):
    def __init__(self) -> None:
        self.host_address = f"https://{os.getenv('SERVER_HOST')}:8081"

    def get_mark_scheme(self, module_name: str, exercise_name: str) -> str:
        response = requests.get(
            f"{self.host_address}/modules/{module_name}/exercises/{exercise_name}/mark_scheme",
            headers={"Authorization": os.getenv("SERVER_API_KEY")},
            verify=False,
            timeout=10,
        )
        response.raise_for_status()
        return str(response.json()["mark_scheme"])

    def get_response(self, request: GPTRequest) -> str:
        try:
            response = requests.post(
                f"{self.host_address}/gpt",
                headers={
                    "Content-Type": "application/json",
                    "Authorization": os.getenv("SERVER_API_KEY"),
                },
                json=request.serialised,
                verify=False,
                timeout=10,
            )
            response.raise_for_status()
        except Exception as _:
            response = requests.post(
                f"{self.host_address}/gpt",
                headers={
                    "Content-Type": "application/json",
                    "Authorization": os.getenv("SERVER_API_KEY"),
                },
                json=request.serialised,
                verify=False,
                timeout=10,
            )
            response.raise_for_status()
        return str(response.json()["response"])



import sys

from marker_tool.interpreter import Interpreter
from marker_tool.utils import write_json_to_file


def main() -> None:
    # Check for correct number of arguments.
    if len(sys.argv) != 4:
        raise ValueError(
            (
                "Insufficient arguments! Expected 3 arguments <module_name> "
                f"<exercise_name> <code_path> but got {len(sys.argv) - 1}"
            )
        )

    # Mark code and write feedback.

    interpreter = Interpreter(sys.argv[1], sys.argv[2], sys.argv[3], debug_mode=True)
    interpreter.run()

    marks = interpreter.get_marks()
    feedback = interpreter.get_feedback()

    write_json_to_file("./comments.json", feedback)
    write_json_to_file("./marks.json", marks)

    interpreter.print_warnings()


if __name__ == "__main__":
    main()
